
import javafx.application.Application;
import static javafx.application.Application.launch;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.PasswordField;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Modality;
import javafx.stage.Stage;

/**
 * A sub class to Stage(!) representing a dialog window, i.e. a temporary window
 * used for e.g. displaying a form. The dialog window is set modal by the
 * constructor; a call to stage.showAndWait also makes it blocking (i.e. the
 * user can only interact with the window (not the parent window) and the
 * execution of the code showing the dialog window halts, until the dialog is
 * closed).
 *
 * The calling application can get the data, in this case a UserInfo object, by
 * calling getData().
 */
class LogOnDialogStage extends Stage {

    private final TextField userNameField;
    private final PasswordField passwordField;

    private UserInfo userInfo = null;

    // constructor, initializing the ui stuff...
    LogOnDialogStage() {
        super(); // super constructor
        this.initModality(Modality.WINDOW_MODAL); // set modal

        userNameField = new TextField();
        passwordField = new PasswordField();

        GridPane form = new GridPane();
        form.setVgap(5);
        form.setHgap(5);
        form.setPadding(new Insets(10, 10, 10, 10));
        form.add(new Label("User name"), 0, 0);
        form.add(userNameField, 1, 0);
        form.add(new Label("Password"), 0, 1);
        form.add(passwordField, 1, 1);

        Button okButton = new Button("Log on");
        Button cancelButton = new Button("Cancel");
        EventHandler buttonHandler = new EventHandler<ActionEvent>() {

            @Override
            public void handle(ActionEvent event) {
                // TODO: check if form data is complete and valid
                if (event.getSource().equals(okButton)) {
                    userInfo = new UserInfo(
                            userNameField.getText(),
                            passwordField.getText());
                    clearFields();
                    System.out.println("Ok"); // debug
                } else if (event.getSource().equals(cancelButton)) {
                    userInfo = null;
                    System.out.println("Cancel"); // debug
                }

                // hide the dialog window
                LogOnDialogStage.this.hide();
            }
        };
        okButton.setOnAction(buttonHandler);
        cancelButton.setOnAction(buttonHandler);
        HBox buttonPane = new HBox(5);
        buttonPane.setAlignment(Pos.CENTER);
        buttonPane.setPadding(new Insets(10, 10, 10, 10));
        buttonPane.getChildren().addAll(okButton, cancelButton);

        BorderPane root = new BorderPane();
        root.setCenter(form);
        root.setBottom(buttonPane);

        Scene scene = new Scene(root);
        this.setResizable(false);
        this.setTitle("Log on");
        this.setScene(scene);
    }

    UserInfo getData() {
        UserInfo temp = userInfo;
        userInfo = null;
        return temp;
    }

    void clearFields() {
        userNameField.setText("");
        passwordField.setText("");
    }
}

/**
 * A simple representation of some data, to be "transferred" from the dialog to
 * the application when the call dialog.ShowAndWait() returns.
 */
class UserInfo {

    private final String name;
    private final String password;

    UserInfo(String name, String password) {
        this.name = name;
        this.password = password;
    }

    String getName() {
        return name;
    }

    String getPassword() {
        return password;
    }
}

/**
 * The application, implementing the main window (stage), and an event handler
 * calling dialog.showAndWait(), at line 163-180, a blocking call.
 */
public class DialogUsingJavaFxStage extends Application {

    private LogOnDialogStage logOnDialog;

    @Override
    public void start(Stage primaryStage) {

        logOnDialog = new LogOnDialogStage(); // modal by default
        logOnDialog.initOwner(primaryStage);

        VBox root = new VBox(20);
        root.setAlignment(Pos.CENTER);
        createApplicationUI(root);

        Scene scene = new Scene(root, 400, 200);

        primaryStage.setTitle("Hello World!");
        primaryStage.setScene(scene);
        primaryStage.show();
    }

    private void createApplicationUI(VBox root) {
        final Text infoText = new Text("- - -");
        infoText.setFont(Font.font("Consolas", 14));

        Button logOnButton = new Button("Log on");
        logOnButton.setOnAction(new EventHandler<ActionEvent>() {

            @Override
            public void handle(ActionEvent event) {

                logOnDialog.showAndWait(); // blocking

                // att this point, showAndWait has returned, 
                // time to call getData
                UserInfo user = logOnDialog.getData();

                if (user != null) {
                    infoText.setText(
                            "Your password is '"
                            + user.getPassword()
                            + "' - Wohahaha!");
                } else {
                    infoText.setText("Canceled");
                }
            }
        });

        root.getChildren().addAll(logOnButton, infoText);
    }

    public static void main(String[] args) {
        launch(args);
    }
}
